<?php

namespace Drupal\style_selector\Element;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element\FormElement;
use Drupal\Component\Utility\Html;

/**
 * Renders the style_selector form element.
 *
 * @FormElement("style_selector")
 *
 * Custom form element properties:
 * - #style_type: (string) Whether the options are a list of CSS classes or
 *   CSS colors. Possible values:
 *   - css_class (default)
 *   - css_color
 * - #ui_style: (string) The UI style of the widget form element. Possible
 *   values:
 *   - round (default)
 *   - round-large
 *   - rect
 *   - rect-large
 * - #styles: (array) Array of value => label pairs that will be transformed
 *   into available options for the element.
 */
class StyleSelector extends FormElement {

  /**
   * {@inheritdoc}
   */
  public function getInfo() {
    $class = get_class($this);
    return [
      '#input' => TRUE,
      '#options' => [],
      '#style_type' => 'css_class',
      '#ui_style' => 'round',
      '#styles' => [],
      '#process' => [
        [$class, 'processFormElement'],
        [$class, 'processGroup'],
      ],
      '#pre_render' => [
        [$class, 'preRenderGroup'],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function processFormElement(array &$element, FormStateInterface $form_state, &$complete_form) {
    // Ensure multiple Style Selector child groups have unique names.
    $element['#tree'] = TRUE;

    // Add the radios.
    $element['style_selector'] = [
      '#prefix' => '<div class="style-selector-ui style-selector-ui--' . $element['#ui_style'] . '">',
      '#suffix' => '</div>',
      '#type' => 'radios',
      '#style_selector' => TRUE,
      '#required' => $element['#required'],
      '#default_value' => $element['#default_value'] ?? NULL,
      '#title' => $element['#title'],
      '#input' => $element['#input'],
    ];

    if (empty($element['#styles'])) {
      return $element;
    }

    $styles = $element['#styles'];

    // Add the none/transparent 'style' if the field is not required.
    if (!$element['#required']) {
      $empty_option = ['none' => t('None')];
      $styles = array_merge($empty_option, $element['#styles']);
    }

    foreach ($styles as $style_value => $style_label) {
      // Skip if the item is missing label or value.
      if (empty($style_value) || empty($style_label)) {
        continue;
      }

      $option = '';
      if ($style_value === 'none') {
        $option = NULL;
      }
      elseif ($element['#style_type'] === 'css_class') {
        $option = Html::getClass($style_value);
      }
      elseif ($element['#style_type'] === 'css_color') {
        if ($valid_color = \Drupal::service('style_selector.css_color')->getFormSafeColorValue($style_value)) {
          $option = $valid_color;
        }
        else {
          \Drupal::logger('style_selector')->info(t(
            'Option <strong>:value|:label</strong> in <strong>:id</strong> was ignored. :value is not a valid CSS color.',
            [
              ':value' => $style_value,
              ':label' => $style_label,
              ':id' => end($element['#parents']),
            ]
          ));
          continue;
        }
      }
      else {
        // Invalid #style_type.
        continue;
      }

      // Build the option element (individual radio).
      $element['style_selector']['#options'][$option] = $style_label;
      $element['style_selector'][$option]['#title'] = $style_label;
      $element['style_selector'][$option]['#attributes']['class'][] = "style-selector-ui-option";

      // Add properties for preprocessing.
      $element['style_selector'][$option]['#style_selector'] = TRUE;
      $element['style_selector'][$option]['#style_type'] = $element['#style_type'];
      $element['style_selector'][$option]['#style_value'] = $style_value;
    }

    // Copy the options to the parent element to ensure proper validation.
    $element['#options'] = $element['style_selector']['#options'];

    // Add the base form element (ui) styles.
    $element['#attached']['library'][] = 'style_selector/style_selector_ui';

    // Add tweak libs for gin and adminimal_theme.
    $theme = \Drupal::service('theme.manager')->getActiveTheme()->getName();
    if (in_array($theme, ['gin', 'adminimal_theme'])) {
      $element['#attached']['library'][] = 'style_selector/style_selector_ui_' . $theme;
    }

    // Attach configured libraries.
    $module_config = \Drupal::config('style_selector.settings');
    $libraries = array_merge($module_config->get('admin_libraries') ?? [], $module_config->get('shared_libraries') ?? []);
    foreach ($libraries as $library) {
      $element['#attached']['library'][] = $library;
    }

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public static function valueCallback(&$element, $input, FormStateInterface $form_state) {
    if ($input !== FALSE) {
      // When there's user input (including NULL), return it as the value.
      // However, if NULL is submitted, FormBuilder::handleInputElement() will
      // apply the default value, and we want that validated against #options
      // unless it's empty. (An empty #default_value, such as NULL or FALSE, can
      // be used to indicate that no radio button is selected by default.)
      if (!isset($input) && !empty($element['#default_value'])) {
        $element['#needs_validation'] = TRUE;
      }
      // If $input is an array, return the first element, which is the value
      // chosen in the radios (if any).
      return (is_array($input)) ? reset($input) : $input;
    }
    else {
      // For default value handling, simply return #default_value. Additionally,
      // for a NULL default value, set #has_garbage_value to prevent
      // FormBuilder::handleInputElement() converting the NULL to an empty
      // string, so that code can distinguish between nothing selected and the
      // selection of a radio button whose value is an empty string.
      $value = $element['#default_value'] ?? NULL;
      if (!isset($value)) {
        $element['#has_garbage_value'] = TRUE;
      }
      return $value;
    }
  }

}
